using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;
using Syncfusion.Drawing;

namespace InPlaceRichTextCellSample
{

	#region cell model class

	public class InPlaceRichTextCellModel : GridTextBoxCellModel
	{
		private RichTextBox rtb;

		public InPlaceRichTextCellModel(GridModel grid)
			: base(grid)
		{
			rtb = new RichTextBox();

			rtb.AutoWordSelection = false;
			rtb.ScrollBars = RichTextBoxScrollBars.None;
			rtb.Location = new Point(-1000, -1000);
			rtb.Visible = false;
		}

		private InPlaceRichTextCellRenderer renderer = null;
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			renderer = new InPlaceRichTextCellRenderer(control, this);
			return renderer;
		}
	
		
		protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
		{
			//don't size col width...
			//  'in versions >= 2.0.6.0, use WinFormsUtils.MeasureSampleWString instead of GridCellModelBase.MeasureSampleWString
			// Size clientSize = new Size(this.Grid.ColWidths[colIndex], WinFormsUtils.MeasureSampleWString(g, style.GdipFont).Height);
			Size clientSize = new Size(this.Grid.ColWidths[colIndex], GridCellModelBase.MeasureSampleWString(g, style.GdipFont).Height);
			
			rtb.Width = clientSize.Width - style.BorderMargins.Left - style.BorderMargins.Right - this.ButtonBarSize.Width;
			
			if (Grid.ActiveGridView == null || !Grid.ActiveGridView.CurrentCell.HasCurrentCellAt(rowIndex, colIndex)
				|| Grid.ActiveGridView.CurrentCell.StaticDrawing)
			{
				if(style.CellValue != null)
				{	
					rtb.Rtf = style.CellValue.ToString();
				}
				else
				{
					rtb.Rtf = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033\viewkind4\uc1\pard\f0\fs17\par}";
				}
			}
			else
				rtb.Rtf = ((RichTextBox)this.renderer.TextBox).Rtf;
			//insert a /par so we get top of next line
			rtb.SelectionLength = 0;//rtb.TextLength;
			rtb.SelectionLength = 0;
			rtb.SelectedRtf = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033\viewkind4\uc1\pard\f0\fs17\par}";
			
			int height =  rtb.GetPositionFromCharIndex(rtb.TextLength).Y  + style.BorderMargins.Top + style.BorderMargins.Bottom + rtb.SelectionFont.Height / 2;

			return new Size(this.Grid.ColWidths[colIndex], height);
		}
	}

	#endregion

	#region cell renderer class

	public class InPlaceRichTextCellRenderer : GridTextBoxCellRenderer 
	{
		//used for in place editing
		GridTextBoxControl activeRichTextBox;
		RichTextBox staticRichTextBox;

		//used to display context menu for active edit control
		//public so the user can modify it 
		public ContextMenu RtbContextMenu;

		public InPlaceRichTextCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{

			SupportsFocusControl = true;

			//remove the button
			//this.Model.ButtonBarSize = new Size(1,1);

			activeRichTextBox = this.TextBox as GridTextBoxControl;//new GridTextBoxControl(this);
			staticRichTextBox = new RichTextBox();
			staticRichTextBox.AutoWordSelection = false;

			(activeRichTextBox as RichTextBox).SelectAll();
			(activeRichTextBox as RichTextBox).AutoWordSelection = false;

			activeRichTextBox.BorderStyle = BorderStyle.None;
			staticRichTextBox.BorderStyle = BorderStyle.None;
			//activeRichTextBox.TextChanged += new EventHandler(activeRichTextBox_TextChanged);

			RtbContextMenu = new ContextMenu(new MenuItem[]
												{
													new MenuItem("Copy", new EventHandler(copyClick)),
													new MenuItem("Paste", new EventHandler(pasteClick)),
													new MenuItem("-"),
													new MenuItem("Change Font", new EventHandler(fontClick)) 
												});

			activeRichTextBox.ContextMenu = RtbContextMenu;
			activeRichTextBox.AutoWordSelection = false;
			RtbContextMenu.Popup += new EventHandler(contextMenu_Popup);
		}

		#region Context menu handlers

		private void contextMenu_Popup(object sender, EventArgs e)
		{
			RtbContextMenu.MenuItems[0].Enabled = CanCopy();
			RtbContextMenu.MenuItems[1].Enabled = CanPaste();
		}
		private void copyClick(object sender, EventArgs e)
		{
			Copy();
		}

		private void pasteClick(object sender, EventArgs e)
		{
			Paste();
		}

		private void fontClick(object sender, EventArgs e)
		{
			FontDialog fontDialog1 = new FontDialog();
			fontDialog1.ShowColor = true;
			fontDialog1.Font = activeRichTextBox.SelectionFont;
			fontDialog1.Color = activeRichTextBox.SelectionColor;
			this.CurrentCell.ConfirmChanges();
		
			if(fontDialog1.ShowDialog() != DialogResult.Cancel )
			{
				activeRichTextBox.Focus();
				activeRichTextBox.SelectionFont = fontDialog1.Font;
				activeRichTextBox.SelectionColor = fontDialog1.Color;
			}
//			else
//				activeRichTextBox.Focus();
		}
		#endregion

		#region helper methods for the renderer class

		//used to set cell changes flag
		private void activeRichTextBox_TextChanged(object sender, EventArgs e)
		{
			this.Grid.CurrentCell.IsModified = true;
		}

		void AssignRtf(RichTextBox rtb, string rtf)
		{
			if (Syncfusion.Drawing.RichTextPaint.IsValidRtf(rtf))
				rtb.Rtf = rtf;
			else 
				rtb.Text = rtf;
		}

		protected void FixControlParent(Control control)
		{
			if (control.Parent != Grid || !control.Visible)
			{
				control.Location = new Point(10000, 10000);

				control.CausesValidation = false;
				control.Anchor = AnchorStyles.None;
				control.Dock = DockStyle.None;
				Grid.GetWindow().Controls.Add(control);
				control.Visible = true;
				SetControl(control);
			}
		}

		#endregion

		#region base class overrides

		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			base.OnInitialize(rowIndex, colIndex);
			ControlValue = this.Grid.Model[rowIndex, colIndex].CellValue;
			FixControlParent(activeRichTextBox);
			if(this.ControlValue == null)
				this.ControlValue = "";
			AssignRtf(activeRichTextBox, this.ControlValue.ToString());
			
			this.Grid.CurrentCell.IsModified = false;
			
		}

		public override bool CanPaste()
		{ 
			bool ok = false;
			foreach(string s in Clipboard.GetDataObject().GetFormats())
			{
				if(s == "Rich Text Format Without Objects")
				{
					ok = true;
					break;
				}
			}
			return ok;
		}

		public override bool CanCopy()
		{ 
			return  activeRichTextBox.SelectionLength != 0;
			
		}

		public override bool Copy()
		{ 
			if (this.HasFocusControl && this.activeRichTextBox.SelectionLength == 0)
				return false;
			if (CurrentCell.HasCurrentCellAt(RowIndex, ColIndex) && HasControlValue)
			{
				this.activeRichTextBox.Copy();
			}
			return true; 
		}

		public override bool Paste()
		{ 
			if (this.activeRichTextBox.ReadOnly)
				return false;

			if (CurrentCell.HasCurrentCellAt(RowIndex, ColIndex) && HasControlValue)
			{
				this.activeRichTextBox.Paste();
				return true;
			}
			return false;
		}
	
		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style) 
		{
			Rectangle textRectangle = RemoveMargins(clientRectangle, style);

			if (CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && 
				HasControlValue && ShouldDrawFocused(rowIndex, colIndex) && CurrentCell.IsEditing)
			{
				// Position current grid
				activeRichTextBox.Bounds = textRectangle;//clientRectangle;
				activeRichTextBox.BackColor = style.BackColor;
				if(!activeRichTextBox.Focused)
					activeRichTextBox.Focus();

			}
			else
			{
				string rtf;
			
				if (CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && CurrentCell.IsModified && HasControlValue)
					rtf = ControlValue.ToString();
				else
					rtf = style.Text;

				Rectangle clipBounds = GetCellBoundsCore(rowIndex, colIndex, true);
				Color backColor = Color.FromArgb(255, style.Interior.BackColor);
				textRectangle = new Rectangle(textRectangle.X + 1, textRectangle.Y, textRectangle.Width - 3, textRectangle.Height);
				
				//in versions >=  2.0.6.0, use
				//RichTextPaint.DrawRichText(g, staticRichTextBox, rtf, Grid.PrintingMode, Grid.GridBounds, textRectangle, 
				//	clipBounds, backColor, style.WrapText, 100, Grid.IsRightToLeft());
			
				RichTextPaint.DrawRichText(g, staticRichTextBox, rtf, Grid.PrintingMode, Grid.GridBounds, textRectangle, 
					clipBounds, backColor, style.WrapText, 100);
			}
		}

		protected override bool OnSaveChanges()
		{
			if (HasControlValue)
			{
				Grid.Model[RowIndex, ColIndex].CellValue = this.activeRichTextBox.Rtf;
				this.Grid.CurrentCell.IsModified = false;
			}
			return true;
		}
		
		protected override void OnClick(int rowIndex, int colIndex, MouseEventArgs e)
		{
			if (this.Grid.CurrentCell.HasCurrentCellAt(rowIndex, colIndex))
			{
				// Activate textbox and show caret 
				GridCellLayout layout = GetCellLayout(rowIndex, colIndex, Grid.Model[rowIndex, colIndex]);
				bool clickOnCell = layout.ClientRectangle.Contains(new Point(e.X, e.Y));
			
				bool beginEdit = false;
				if (clickOnCell && (Grid.Model.Options.ActivateCurrentCellBehavior & (GridCellActivateAction.ClickOnCell)) != 0
					|| (Grid.Model.Options.ActivateCurrentCellBehavior & GridCellActivateAction.SetCurrent) != 0)
					beginEdit = true;

				if (beginEdit)
				{
					CurrentCell.BeginEdit();
					if ((Grid.Model.Options.ActivateCurrentCellBehavior & GridCellActivateAction.SelectAll) == 0)
					{
						if (e.Button == MouseButtons.Left 
							&& (Grid.Model.Options.ActivateCurrentCellBehavior & GridCellActivateAction.PositionCaret) != 0)
						{
							Grid.Update();
							Point loc = layout.TextRectangle.Location;
							Point p = new Point(e.X-loc.X, e.Y-loc.Y);
							Syncfusion.Drawing.ActiveXSnapshot.FakeLeftMouseClick(this.activeRichTextBox, p);
						}
						else
						{
							Point loc = layout.TextRectangle.Location;
							Point p = new Point(e.X-loc.X, e.Y-loc.Y);
							ControlMouseDown(this, new MouseEventArgs(e.Button, e.Clicks, p.X, p.Y, 0));
						}
					}
					else
					{
						Point loc = layout.TextRectangle.Location;
						Point p = new Point(e.X-loc.X, e.Y-loc.Y);
						ControlMouseDown(this, new MouseEventArgs(e.Button, e.Clicks, p.X, p.Y, 0));
					}
				}
			}
		
			base.OnClick(rowIndex, colIndex, e);
		}
		#endregion
	}
	#endregion

}
